@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/**
+/*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -37,6 +37,15 @@
 #include "egg-recent-util.h"
 #include "egg-recent-item.h"
 
+#ifndef EGG_COMPILATION
+#include <glib/gi18n.h>
+#else
+#define _(x) (x)
+#define N_(x) (x)
+#endif
+
+#define DEFAULT_LABEL_WIDTH 30
+
 struct _EggRecentViewGtk {
 	GObject parent_instance;	/* We emit signals */
 
@@ -52,9 +61,7 @@ struct _EggRecentViewGtk {
 
 	gboolean show_icons;
 	gboolean show_numbers;
-#ifndef USE_STABLE_LIBGNOMEUI
-	GnomeIconTheme *theme;
-#endif
+	GtkIconTheme *theme;
 
 	GtkTooltips *tooltips;
 	EggRecentViewGtkTooltipFunc tooltip_func;
@@ -63,6 +70,8 @@ struct _EggRecentViewGtk {
 	EggRecentModel *model;
 	GConfClient *client;
 	GtkIconSize icon_size;
+
+	gint label_width;
 };
 
 
@@ -85,11 +94,26 @@ enum {
 	PROP_MENU,
 	PROP_START_MENU_ITEM,
 	PROP_SHOW_ICONS,
-	PROP_SHOW_NUMBERS
+	PROP_SHOW_NUMBERS,
+	PROP_LABEL_WIDTH
 };
 
 static guint view_signals[LAST_SIGNAL] = { 0 };
 
+static GObjectClass *parent_class;
+
+/* mark a menu item, so that we know which ones we own */
+static void
+egg_recent_view_gtk_set_item_tag (EggRecentViewGtk *view,
+				  GtkMenuItem      *menu_item)
+{
+	g_return_if_fail (EGG_IS_RECENT_VIEW_GTK (view));
+	g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
+
+	g_object_set_data (G_OBJECT (menu_item),
+			   view->uid,
+			   GINT_TO_POINTER (1));
+}
 
 static void
 egg_recent_view_gtk_clear (EggRecentViewGtk *view)
@@ -99,16 +123,17 @@ egg_recent_view_gtk_clear (EggRecentViewGtk *view)
 	GObject *menu_item;
 	gint *menu_data=NULL;
 
-	g_return_if_fail (view->menu != NULL);
+	if (view->menu == NULL)
+		return;
 
 	menu_children = gtk_container_get_children (GTK_CONTAINER (view->menu));
 
 	p = menu_children;
 	while (p != NULL) {
-		menu_item = (GObject *)p->data;
+		menu_item = G_OBJECT (p->data);
 
-		menu_data = (gint *)g_object_get_data (menu_item,
-						       view->uid);
+		menu_data = (gint *) g_object_get_data (menu_item,
+						        view->uid);
 	
 		if (menu_data) {
 			gtk_container_remove (GTK_CONTAINER (view->menu),
@@ -190,14 +215,11 @@ egg_recent_view_gtk_new_separator (EggRecentViewGtk *view)
 	
 	retval = gtk_separator_menu_item_new ();
 
-	/**
+	/*
 	 * this is a tag so we can distinguish our menu items
 	 * from others that may be in the menu.
 	 */
-	g_object_set_data (G_OBJECT (retval),
-			   view->uid,
-			   GINT_TO_POINTER (1));
-
+	egg_recent_view_gtk_set_item_tag (view, GTK_MENU_ITEM (retval));
 
 	gtk_widget_show (retval);
 
@@ -213,6 +235,7 @@ egg_recent_view_gtk_new_menu_item (EggRecentViewGtk *view,
 	EggRecentViewGtkMenuData *md;
 	gchar *mime_type;
 	GtkWidget *image;
+	GtkWidget *label;
 	GdkPixbuf *pixbuf;
 	gchar *text;
 	gchar *short_name;
@@ -272,6 +295,10 @@ egg_recent_view_gtk_new_menu_item (EggRecentViewGtk *view,
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
 				       image);
 
+	label = GTK_BIN (menu_item)->child;
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (GTK_LABEL (label), view->label_width);
+
 	md = g_new0 (EggRecentViewGtkMenuData, 1);
 	md->view = view;
 	md->item = egg_recent_item_ref (item);
@@ -285,20 +312,49 @@ egg_recent_view_gtk_new_menu_item (EggRecentViewGtk *view,
 	g_free (mime_type);
 	g_free (text);
 
-	/**
+	/*
 	 * this is a tag so we can distinguish our menu items
 	 * from others that may be in the menu.
 	 */
-	g_object_set_data (G_OBJECT (menu_item),
-			   view->uid,
-			   GINT_TO_POINTER (1));
-
+	egg_recent_view_gtk_set_item_tag (view, GTK_MENU_ITEM (menu_item));
 
 	gtk_widget_show (menu_item);
 
 	return menu_item;
 }
 
+static void
+egg_recent_view_gtk_create_tooltip (EggRecentViewGtk *view,
+				    GtkWidget        *menu_item,
+				    EggRecentItem    *recent_item)
+{
+	gchar *name, *tip_text;
+	
+	g_return_if_fail (EGG_IS_RECENT_VIEW_GTK (view));
+	g_return_if_fail (GTK_IS_WIDGET (menu_item));
+	g_return_if_fail (recent_item != NULL);
+
+	if (!view->tooltips)
+		return;
+
+	name = egg_recent_item_get_uri_for_display (recent_item);
+	if (!name)
+		return;
+
+	tip_text = g_strdup_printf (_("Open '%s'"), name);
+	if (!tip_text) {
+		g_free (name);
+		return;
+	}
+
+	gtk_tooltips_set_tip (view->tooltips, menu_item,
+			      tip_text,
+			      NULL);
+	
+	g_free (tip_text);
+	g_free (name);
+}
+
 static void
 egg_recent_view_gtk_add_to_menu (EggRecentViewGtk *view,
 				 EggRecentItem *item,
@@ -313,19 +369,30 @@ egg_recent_view_gtk_add_to_menu (EggRecentViewGtk *view,
 
 	menu_offset = egg_recent_view_gtk_find_menu_offset (view);
 
-	if (item != NULL)
+	if (item != NULL) {
 		menu_item = egg_recent_view_gtk_new_menu_item (view, item, display);
+		
+		/* if present, use the custom tooltip function;
+		 * otherwise, use ours (which has been "borrowed"
+		 * from GEdit)
+		 */
+		if (view->tooltip_func != NULL) {
+			view->tooltip_func (view->tooltips,
+					    menu_item,
+					    item,
+					    view->tooltip_func_data);
+		}
+		else {
+			egg_recent_view_gtk_create_tooltip (view, menu_item, item);
+		}
+	}
 	else
 		menu_item = egg_recent_view_gtk_new_separator (view);
 
-	if (view->tooltip_func != NULL && menu_item != NULL) {
-		view->tooltip_func (view->tooltips, menu_item,
-				    item, view->tooltip_func_data);
-	}
-	
 	if (menu_item)
-		gtk_menu_shell_insert (GTK_MENU_SHELL (view->menu), menu_item,
-			       menu_offset+index);
+		gtk_menu_shell_insert (GTK_MENU_SHELL (view->menu),
+				       menu_item,
+				       menu_offset + index);
 }
 
 static void
@@ -336,7 +403,8 @@ egg_recent_view_gtk_set_list (EggRecentViewGtk *view, GList *list)
 	gint display=1;
 	gint index=1;
 
-	g_return_if_fail (view);
+	if (view->menu == NULL)
+		return;
 
 	egg_recent_view_gtk_clear (view);
 
@@ -360,13 +428,34 @@ egg_recent_view_gtk_set_list (EggRecentViewGtk *view, GList *list)
 		egg_recent_view_gtk_add_to_menu (view, NULL, display, index);
 }
 
+static void
+egg_recent_view_gtk_set_empty_list (EggRecentViewGtk *view)
+{
+	gboolean is_menu_embedded = FALSE;
+
+	egg_recent_view_gtk_clear (view);
+	
+	is_menu_embedded = view->trailing_sep || (egg_recent_view_gtk_find_menu_offset (view) > 0);
+	if (!is_menu_embedded) {
+		GtkWidget *dummy_item;
+
+		dummy_item = gtk_menu_item_new_with_label (_("Empty"));
+		gtk_widget_set_sensitive (dummy_item, FALSE);
+		gtk_menu_shell_insert (GTK_MENU_SHELL (view->menu), dummy_item, 0);
+		gtk_widget_show (dummy_item);
+
+		/* we own this item */
+		egg_recent_view_gtk_set_item_tag (view, GTK_MENU_ITEM (dummy_item));
+	}
+}
+
 static void
 model_changed_cb (EggRecentModel *model, GList *list, EggRecentViewGtk *view)
 {
 	if (list != NULL)
 		egg_recent_view_gtk_set_list (view, list);
 	else
-		egg_recent_view_gtk_clear (view);
+		egg_recent_view_gtk_set_empty_list (view);
 }
 
 static EggRecentModel *
@@ -453,6 +542,10 @@ egg_recent_view_gtk_set_property (GObject *object,
 			egg_recent_view_gtk_show_numbers (view,
 					g_value_get_boolean (value));
 		break;
+		case PROP_LABEL_WIDTH:
+		        egg_recent_view_gtk_set_label_width (view,
+					g_value_get_int (value));
+		break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -481,6 +574,9 @@ egg_recent_view_gtk_get_property (GObject *object,
 		case PROP_SHOW_NUMBERS:
 			g_value_set_boolean (value, view->show_numbers);
 		break;
+		case PROP_LABEL_WIDTH:
+			g_value_set_int (value, view->label_width);
+		break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -495,16 +591,16 @@ egg_recent_view_gtk_finalize (GObject *object)
 	g_signal_handler_disconnect (G_OBJECT (view->model),
 				     view->changed_cb_id);
 
+	egg_recent_view_gtk_clear (view);
+
 	g_free (view->uid);
 
-	g_object_unref (view->menu);
 	g_object_unref (view->model);
-#ifndef USE_STABLE_LIBGNOMEUI
-	g_object_unref (view->theme);
-#endif
 	g_object_unref (view->client);
 
 	g_object_unref (view->tooltips);
+
+	parent_class->finalize (object);
 }
 
 static void
@@ -514,6 +610,8 @@ egg_recent_view_gtk_class_init (EggRecentViewGtkClass * klass)
 
 	object_class = G_OBJECT_CLASS (klass);
 
+	parent_class = g_type_class_peek_parent (klass);
+
 	object_class->set_property = egg_recent_view_gtk_set_property;
 	object_class->get_property = egg_recent_view_gtk_get_property;
 	object_class->finalize     = egg_recent_view_gtk_finalize;
@@ -558,6 +656,16 @@ egg_recent_view_gtk_class_init (EggRecentViewGtkClass * klass)
 					   TRUE,
 					   G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_LABEL_WIDTH,
+					 g_param_spec_int ("label-width",
+					   "Label Width",
+					   "The desired width of the menu label, in characters",
+					   -1,
+					   G_MAXINT,
+					   DEFAULT_LABEL_WIDTH,
+					   G_PARAM_READWRITE));
+
 	klass->activate = NULL;
 }
 
@@ -585,14 +693,12 @@ show_menus_changed_cb (GConfClient *client,
 
 }
 
-#ifndef USE_STABLE_LIBGNOMEUI
 static void
-theme_changed_cb (GnomeIconTheme *theme, EggRecentViewGtk *view)
+theme_changed_cb (GtkIconTheme *theme, EggRecentViewGtk *view)
 {
 	if (view->model != NULL)
 		egg_recent_model_changed (view->model);
 }
-#endif
 
 static void
 egg_recent_view_gtk_init (EggRecentViewGtk * view)
@@ -617,12 +723,9 @@ egg_recent_view_gtk_init (EggRecentViewGtk * view)
 	view->trailing_sep = FALSE;
 
 	view->uid = egg_recent_util_get_unique_id ();
-#ifndef USE_STABLE_LIBGNOMEUI
-	view->theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (view->theme, TRUE);
+	view->theme = gtk_icon_theme_get_default ();
 	g_signal_connect_object (view->theme, "changed",
 				 G_CALLBACK (theme_changed_cb), view, 0);
-#endif
 	view->tooltips = gtk_tooltips_new ();
 	g_object_ref (view->tooltips);
 	gtk_object_sink (GTK_OBJECT (view->tooltips));
@@ -630,6 +733,8 @@ egg_recent_view_gtk_init (EggRecentViewGtk * view)
 	view->tooltip_func_data = NULL;
 
 	view->icon_size = GTK_ICON_SIZE_MENU;
+
+	view->label_width = DEFAULT_LABEL_WIDTH;
 }
 
 void
@@ -680,6 +785,23 @@ egg_recent_view_gtk_set_tooltip_func (EggRecentViewGtk *view,
 		egg_recent_model_changed (view->model);
 }
 
+void
+egg_recent_view_gtk_set_label_width (EggRecentViewGtk *view,
+				     gint              chars)
+{
+	g_return_if_fail (EGG_IS_RECENT_VIEW_GTK (view));
+
+	view->label_width = chars;
+}
+
+gint
+egg_recent_view_gtk_get_label_width (EggRecentViewGtk *view)
+{
+	g_return_val_if_fail (EGG_IS_RECENT_VIEW_GTK (view), -1);
+
+	return view->label_width;
+}
+
 /**
  * egg_recent_view_gtk_set_menu:
  * @view: A EggRecentViewGtk object.
@@ -695,13 +817,16 @@ egg_recent_view_gtk_set_menu (EggRecentViewGtk *view,
 {
 	g_return_if_fail (view);
 	g_return_if_fail (EGG_IS_RECENT_VIEW_GTK (view));
-	g_return_if_fail (menu);
 
 	if (view->menu != NULL)
-		g_object_unref (view->menu);
+		g_object_remove_weak_pointer (G_OBJECT (view->menu),
+					      (gpointer *) &view->menu);
 	
 	view->menu = menu;
-	g_object_ref (view->menu);
+
+	if (view->menu != NULL)
+		g_object_add_weak_pointer (G_OBJECT (view->menu),
+					   (gpointer *) &view->menu);
 }
 
 /**
@@ -773,7 +898,6 @@ egg_recent_view_gtk_new (GtkWidget *menu, GtkWidget *start_menu_item)
 
 /**
  * egg_recent_view_gtk_get_type:
- * @:
  *
  * This returns a GType representing a EggRecentViewGtk object.
  *
